<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage API
 * @author     Paul Oehler <poehler@interworx.com>
 * @date       Nov 24, 2008
 */

/**
 * General API function calls.
 *
 * @package    InterWorx
 * @subpackage API
 */
class IWorx_API_General
{

    /**
     * Get the api version.
     *
     * @param mixed $params
     * @return string
     */
    public function apiversion($params)
    {
        API::auth($params);
        return API::VERSION;
    }

    /**
     * Get the iworx version.
     *
     * @param array|struct $params
     * @return string
     */
    public function cpversion($params)
    {
        API::auth($params);
        return Ini::get(Ini::IWORX, 'version');
    }

    /**
     * Get the iworx build.
     *
     * @param array|struct $params
     * @return string
     */
    public function buildnumber($params)
    {
        API::auth($params);
        $version = explode('-', Ini::get(Ini::IWORX, 'rpm.release'));
        $build = $version[1];

        if (!isset($build)) {
            return -1;
        } else {
            return $build;
        }
    }

    /**
     * Get the OS.
     *
     * @param array|struct $params
     * @return string
     */
    public function os($params)
    {
        API::auth($params);
        return php_uname('s');
    }

    /**
     * Get the architecture.
     *
     * @param array|struct $params
     * @return string
     */
    public function arch($params)
    {
        API::auth($params);
        return php_uname('m');
    }

    /**
     * Gets an IP address from the box.
     *
     * @param array|struct $params
     * @return string
     */
    public function getip($params)
    {
        $NodeWorx = API::auth($params, API::AUTH_RETURN_INTERWORX);

        $type = $params['type'];
        $rule = $params['rule'];

        if ($type == '') {
            $type = IWorx_IP::SHAREDIP;
        }
        if ($rule == '') {
            $rule = 'first';
        }

        $DS = new DataSource_Ip_Types();

        if ($DS->isValidValue($type) === false) {
            return API::formatResponse(API::ERROR_INVALID_IP_TYPE);
        }

        if ($NodeWorx->getId() <= 0) {
            IWorxLog::log(
                'no such NodeWorx user for get ip',
                IWorxLog::DEBUG
            );
            return API::formatResponse(API::ERROR_INVALID_NODEWORX_ACCOUNT);
        }

        $ips = [];

        $ipsx = $NodeWorx->getUnusedIPList();
        foreach ($ipsx as $IWorx_IP) {
            if ($IWorx_IP->isAvailable() &&
            $IWorx_IP->getTypeString() === $type &&
            $IWorx_IP->getVersion() === IWorx_IP::VERSION_4 ) {
                $ips[] = $IWorx_IP;
            }
        }

        if (count($ips) === 0) {
            if ($type === IWorx_IP::STATICIP) {
                return API::formatResponse(API::ERROR_NO_STATIC_IPS);
            }
            return( API::formatResponse(
                API::ERROR_NO_SHARED_IPS,
                'No shared ips available'
            ) );
        }

        $ipaddr = false;

        switch ($rule) {
            case 'first':
                $IWorx_IP = array_shift($ips);
                break;

            case 'most':
                $domain_count = 0;
                $IWorx_IP     = null;

                foreach ($ips as $IP) {
                    if ($IP->getDomainCount() >= $domain_count) {
                        $IWorx_IP     = $IP;
                        $domain_count = $IP->getDomainCount();
                    }
                }

                break;

            case 'least':
                $domain_count = pow(2, 31);
                $IWorx_IP     = null;

                foreach ($ips as $IP) {
                    if ($IP->getDomainCount() === 0) {
                        $IWorx_IP = $IP;
                        break;
                    }

                    if ($IP->getDomainCount() < $domain_count) {
                        $IWorx_IP     = $IP;
                        $domain_count = $IP->getDomainCount();
                    }
                }

                break;

            case 'last':
                $IWorx_IP = array_pop($ips);
                break;

            default:
                return( API::formatResponse(API::ERROR_UNKNOWN_IP_RULE, 'Unknown IP Rule') );
        }//end switch

        return $IWorx_IP->getIP();
    }

    /**
     * Retrieve current account bandwidth usage.
     *
     * @param array|struct $params
     * @return array|struct
     */
    public function syncbwdata($params)
    {
        $NodeWorx = API::auth($params, API::AUTH_RETURN_INTERWORX);

        if ($NodeWorx->getId() <= 0) {
            IWorxLog::log(
                'no such NodeWorx user for sync bandwidth',
                IWorxLog::DEBUG
            );
            return API::formatResponse(API::ERROR_INVALID_NODEWORX_ACCOUNT);
        }

        $unit = $params['unit'];

        switch ($unit) {
            case 'byte':
            case 'bytes':
                $divisor = 1;
                break;
            case 'kilobyte':
            case 'kilobytes':
                $divisor = Ini::get(Ini::CONSTANTS, 'kilobyte');
                break;
            case 'megabyte':
            case 'megabytes':
                $divisor = Ini::get(Ini::CONSTANTS, 'megabyte');
                break;
            case 'gigabyte':
            case 'gigabytes':
                $divisor = Ini::get(Ini::CONSTANTS, 'gigabyte');
                break;
            default:
                $divisor = 1;
        }

        $accounts = $NodeWorx->getAccounts(
            $sort = '',
            $data = array( 'bandwidth_usage' )
        );

        $data = [];
        foreach ($accounts as $onepackage) {
            $data[$onepackage['bwdata']['domain']] =
            $onepackage['bwdata']['bandwidth'] / $divisor;
        }

        return $data;
    }
    /**
     * Checks to see if this server has an active license.
     *
     * @param array|struct $params
     * @return integer
     */
    public function is_licenseactive($params)
    {
        API::auth($params);
        $cmd = Ini::get(Ini::IWORX_CRON, 'license') . ' --check-only';
        IWorxExec::exec($cmd, $cmd_result, $cmd_retval, IWorxExec::KILL_STDERR);
        return ($cmd_retval) ? 0 : 1;
    }
}
