<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Paul Oehler <poehler@interworx.com>
 * @date    Sep 22, 2009
 */

/**
 * R1soft integration plugin.
 *
 * @package    InterWorx
 * @subpackage Plugin
 */
class Plugin_R1soft extends Plugin
{

    /**
     * Customizations to siteworx menu can be done here.
     *
     * @param IWorxMenuManager $MenuMan
     */
    public function updateSiteworxMenu(IWorxMenuManager $MenuMan)
    {
        if (!$this->userIsBlocked(IW::SW()->getUniqname())) {
            $new_data = array( 'text'   => 'R1Soft Restore',
                         'url'    => '/siteworx/r1soft',
                         'class'  => 'iw-i-star',
                         'hide'   => '~WEB' );
            $MenuMan->addMenuItemAfter('iw-menu-stats', 'r1soft', $new_data);
        }
    }

    /**
     * True if the provided unixuser is blocked from r1soft access.
     *
     * @param string $unixuser
     * @return boolean
     */
    public function userIsBlocked($unixuser)
    {
        if (in_array($unixuser, (array)$this->getDatastoreVar('blocked_users'))) {
            return true;
        }
        return false;
    }

    /**
     * Block a unixuser from using r1soft.
     *
     * @param string $unixuser
     */
    public function blockUser($unixuser)
    {
        $blocked_users = $this->getDatastoreVar('blocked_users');
        if (!is_array($blocked_users)) {
            $blocked_users = [];
        }
        $blocked_users[] = $unixuser;
        $blocked_users   = array_unique($blocked_users);
        $this->setDatastoreVar('blocked_users', $blocked_users);
    }

    /**
     * Unblock a unixuser from using r1soft.
     *
     * @param string $unixuser
     */
    public function unblockUser($unixuser)
    {
        $blocked_users = $this->getDatastoreVar('blocked_users');
        if (!is_array($blocked_users)) {
            $blocked_users = [];
        }
        $key = array_search($unixuser, $blocked_users);
        if ($key !== false) {
            unset($blocked_users[ $key ]);
            $this->setDatastoreVar('blocked_users', $blocked_users);
        }
    }

    /**
     * Get category.
     *
     * @return string
     */
    public function getCategory()
    {
        return Plugin_Category::BACKUP_RESTORE;
    }

    /**
     * Init cdp2.
     */
    private function _initCdp2()
    {
        $config = '/etc/buagent/agent_config';
        $this->_setVar('agent_config', $config);

        $lines = file($config);
        if (!isset($lines[0])) {
            $reason = "{$config} file empty or unreadable";
            $this->_disable($reason);
            return;
        }

        foreach ($lines as $line_num => $line) {
            $line = rtrim($line);
            if (preg_match('/^DBDIR +(.*)/i', $line, $matches)) {
                $vardb = $matches[ 1 ];
            }
        }

        if (!isset($vardb)) {
            $reason = "Unable to find DBDIR in {$config}";
            $this->_disable($reason);
            return;
        }

        if (!IWorxFileSys::isFile("{$vardb}/serverurl")) {
            $reason = "{$vardb}/serverurl not found";
            $this->_disable($reason);
            return;
        }

        $serverurllines = file($vardb . '/' . 'serverurl');
        $serverurl      = trim($serverurllines[ 0 ]);
        if (!isset($serverurl)) {
            $reason = 'Unable to find the backup server URL.  This may be because ' .
            'the server has not yet been backed up.';
            $this->_disable($reason);
            return;
        }

        if (!IWorxFileSys::isFile("{$vardb}/hostname")) {
            $reason = "{$vardb}/hostname not found";
            $this->_disable($reason);
            return;
        }

        $hostnamelines = file($vardb . '/' . 'hostname');
        $hostname      = $hostnamelines[ 0 ];

        if (!isset($serverurl)) {
            $reason = "{$vardb}/hostname file empty or unreadable";
            $this->_disable($reason);
            return;
        }

        $this->_setVar('serverurl', $serverurl);
        $this->_setVar('hostname', $hostname);
        $this->setDatastoreVar('cdp_ver', '2');
    }

    /**
     * Init cdp3.
     */
    private function _initCdp3()
    {
        $this->setDatastoreVar('cdp_ver', '3');
    }

    /**
     * Get the cdp version.
     *
     * @return string
     */
    public function getCdpVer()
    {
        return $this->getDatastoreVar('cdp_ver');
    }

    /**
     * Init the plugin, disabling if necessary.
     */
    protected function _init()
    {
        $cdp2_config = '/etc/buagent/agent_config';
        $cdp3_config = '/etc/init.d/cdp-agent';
        if (IWorxFileSys::fileExists($cdp2_config)) {
            $this->_initCdp2();
        } elseif (IWorxFileSys::fileExists($cdp3_config)) {
            $this->_initCdp3();
        } else {
            $this->_disable('Could not identify installed r1soft cdp agent version.');
            $this->setDatastoreVar('cdp_ver', '');
        }
    }

    /**
     * Post action.
     *
     * @param string        $ctrl_act Controller_Name:action
     * @param Ctrl_Abstract $Ctrl
     * @param string        $action
     * @param mixed         $params
     */
    public function postAction($ctrl_act, Ctrl_Abstract $Ctrl, $action, $params)
    {
        switch ($ctrl_act) {
            case 'Ctrl_Nodeworx_Siteworx:editCommit';
            case 'Ctrl_Nodeworx_Siteworx:addCommit':
                assert($params instanceof Form);
                $Form = $params;
                if ($Form->getValue('r1soft_access') === '1') {
                    $this->unblockUser($Form->getValue('uniqname'));
                } else {
                    $this->blockUser($Form->getValue('uniqname'));
                }

            default:
          return;
        }
    }

    /**
     * Initialize my edit form.
     *
     * @param Form_NW_Plugins_Edit $Form
     */
    public function initializeMyEditForm(Form_NW_Plugins_Edit $Form)
    {
        $cdp2_config = '/etc/buagent/agent_config';
        $cdp3_config = '/etc/init.d/cdp-agent';
        if (IWorxFileSys::fileExists($cdp2_config)) {
            $this->_initCdp2();
        } elseif (IWorxFileSys::fileExists($cdp3_config)) {
            $this->_initCdp3();
        } else {
            $this->_disable('Could not identify installed r1soft cdp agent version.');
        }

        if ($this->getCdpVer() === '3') {
            $cp_id = $this->getDatastoreVar('cdp_control_panel_id');
            $cdp_server = $this->getDatastoreVar('cdp_server_url');

            $Group = $Form->addGroup('plugin_settings', 'Plugin Settings');
            $Group->addInput(new Input_String('cdp_ver', '3', $enabled = false))
            ->setLabel('CDP Version Detected');
            $Group->addInput(new Input_String('cdp_server_url', $cdp_server))
            ->setLabel('CDP Server URL')
            ->setDescription('Example: https://r1soft001.example.com:8001/');
            $Group->addInput(new Input_String('cdp_control_panel_id', $cp_id))
            ->setLabel('CDP Control Panel ID')
            ->setDescription('Example: sd45d51a-6724-4h31-4gdw-h461cgac13b1');
        } elseif ($this->getCdpVer() === '2') {
            $Group = $Form->addGroup('plugin_settings', 'Plugin Settings');
            $Group->addInput(new Input_String('cdp_ver', '2', $enabled = false))
            ->setLabel('CDP Version Detected');
        } else {
            $Group = $Form->addGroup('plugin_settings', 'Plugin Settings');
            $Group->addInput(new Input_String('cdp_ver', 'Unknown', $enabled = false))
            ->setLabel('CDP Version Detected');
        }//end if
    }

    /**
     * Process my edit form.
     *
     * @param Form_NW_Plugins_Edit $Form
     */
    public function processMyEditForm(Form_NW_Plugins_Edit $Form)
    {
        if ($this->getCdpVer() === '3') {
            $cp_id = $Form->getValue('cdp_control_panel_id');
            $cdp_server = $Form->getValue('cdp_server_url');
            $this->setDatastoreVar('cdp_control_panel_id', $cp_id);
            $this->setDatastoreVar('cdp_server_url', $cdp_server);
        }
    }

    /**
     * Initialize Forms we care about.
     *
     * @param Form $Form
     */
    public function initializeForm(Form $Form)
    {
        $form_class = get_class($Form);

        $ds_type = DataSource_Boolean::YES_NO;
        if ($form_class === 'Form_NW_Siteworx_Add') {
            $default = 1;
            $Form->addGroup('r1soft', 'R1Soft Plugin Settings')
            ->addInput(new Input_Select('r1soft_access', $default))
            ->setDataSource(DataSource_Boolean::factory($ds_type))
            ->setLabel('R1Soft Backup Access');
        }

        if ($form_class === 'Form_NW_Siteworx_Edit') {
            $uniq    = $Form->getInput('uniqname')->getCurrentValue();
            $current = ($this->userIsBlocked($uniq)) ? 0 : 1;
            $Form->addGroup('r1soft', 'R1Soft Plugin Settings')
            ->addInput(new Input_Select('r1soft_access', $current))
            ->setDataSource(DataSource_Boolean::factory($ds_type))
            ->setLabel('R1Soft Backup Access');
        }
    }
}
